# -*- coding: utf-8 -*-

__version__ = '2.4.0'
program_name = 'bio_hansel'
program_summary = 'Subtype microbial genomes using SNV targeting k-mer subtyping schemes.'
program_desc = program_summary + '''
Includes schemes for Salmonella enterica spp. enterica serovar Heidelberg, Enteritidis, Typhi, and Typhimurium subtyping. Also includes a Mycobacterium tuberculosis scheme called 'tb_lineage'. 
Developed by Geneviève Labbé, Peter Kruczkiewicz, Philip Mabon, James Robertson, Justin Schonfeld, Daniel Kein, Marisa A. Rankin, Matthew Gopez, Darian Hole, David Son, Natalie Knox, Chad R. Laing, Kyrylo Bessonov, Eduardo Taboada, Catherine Yoshida, Roger P. Johnson, Gary Van Domselaar and John H.E. Nash.
'''



# -*- coding: utf-8 -*-
import logging
from typing import Dict, List, Set
import re

import attr
from collections import defaultdict

from .parsers import parse_fasta


@attr.s
class SubtypeCounts:
    subtype = attr.ib()
    refpositions = attr.ib(default=None)
    subtype_kmer_count = attr.ib(default=0, validator=attr.validators.instance_of(int))
    positive_kmer_count = attr.ib(default=0, validator=attr.validators.instance_of(int))
    negative_kmer_count = attr.ib(default=0, validator=attr.validators.instance_of(int))
    all_kmer_count = attr.ib(default=0, validator=attr.validators.instance_of(int))

    @subtype.validator
    def _check_subtype(self, attribute, value):
        REGEX_SUBTYPE = re.compile(r'^(\d*[^\d\.]*(\.\d+)*)$')
        if value is None or value == '':
            raise ValueError('Subtype cannot be None or empty string')
        if not REGEX_SUBTYPE.match(value):
            raise ValueError(
                'Invalid subtype specified! The "{}" kmer is not formatted correctly. It must be numbers delimited by "." (periods)'.format(value))
        return value 

    @subtype_kmer_count.validator
    def _check_subtype_kmer_count(self, attribute, value):
        if value == 0:
            raise ValueError('Subtype kmer count cannot be zero!')

    @positive_kmer_count.validator
    def _check_positive_kmer_count(self, attribute, value):
        if value < self.subtype_kmer_count:
            raise ValueError(
                'Subtype {}: Number of all subtype positive kmers (n={}) cannot be less than the number of subtype specific kmers (n={})'.format(
                    self.subtype,
                    value,
                    self.subtype_kmer_count))
        if value > self.all_kmer_count:
            raise ValueError(
                'Subtype {}: Number of all subtype positive kmers (n={}) cannot exceed number of all matching kmers for subtype (n={})'.format(
                    self.subtype,
                    value,
                    self.all_kmer_count))
        if value == 0:
            raise ValueError('Subtype {}: Number of all subtype positive kmers cannot be zero!'.format(self.subtype))


def _kmers(kmers_fasta: str) -> (Dict[str, List[str]], Dict[str, List[str]], Set[int]):
    kmers = defaultdict(list)
    neg_kmers = defaultdict(list)
    sizes = set()
    for h, s in parse_fasta(kmers_fasta):
        sizes.add(len(s))
        _, st = h.split('-')
        if 'negative' not in h:
            kmers[st].append(h)
        else:
            neg_kmers[st].append(h)
    return kmers, neg_kmers, sizes


def subtype_counts(scheme_fasta: str) -> Dict[str, SubtypeCounts]:
    subtype_counts = {}
    kmers, neg_kmers, sizes = _kmers(scheme_fasta)
    if len(sizes) > 1:
        logging.warning('Not all markers in "%s" of the same size! %s', scheme_fasta, sizes)
    n_kmers_total = sum([len(vs) for vs in kmers.values()])
    ks = [x for x in kmers.keys()]
    ks.sort()
    for k in ks:
        st_pos_count_rest = 0
        subtypes_set = {k}
        if len(k) > 1:
            *r, _ = k.split('.')
            for i in range(len(r)):
                sub_k = '.'.join(r[:i + 1])
                subtypes_set.add(sub_k)
                pos_kmers = kmers[sub_k]
                st_pos_count_rest += len(pos_kmers)
        st_count = len(kmers[k])
        st_count_pos = st_pos_count_rest + st_count
        st_neg_count = sum([len(v) for k,v in neg_kmers.items() if k not in subtypes_set])

        subtype_count = SubtypeCounts(subtype=k,
                                      refpositions={int(v.split('-')[0]) for v in kmers[k]},
                                      subtype_kmer_count=st_count,
                                      positive_kmer_count=st_count_pos,
                                      negative_kmer_count=st_neg_count,
                                      all_kmer_count=st_neg_count + st_count_pos)
        subtype_counts[k] = subtype_count
    return subtype_counts



# -*- coding: utf-8 -*-

from typing import Tuple, Optional

import pandas as pd

from ..qc.const import QC
from ..qc.utils import get_conflicting_kmers, get_num_pos_neg_kmers, get_mixed_subtype_kmer_counts
from ..subtype import Subtype
from ..subtyping_params import SubtypingParams


def is_overall_coverage_low(st: Subtype, df: pd.DataFrame, p: SubtypingParams) -> Tuple[Optional[str], Optional[str]]:
    if not st.are_subtypes_consistent \
            or st.subtype is None \
            or not st.is_fastq_input():
        return None, None

    if st.avg_kmer_coverage < p.min_coverage_warning:
        return QC.WARNING, f'Low coverage for all kmers ({st.avg_kmer_coverage:.3f} < {p.min_coverage_warning} expected)'
    return None, None


def is_missing_kmers(st: Subtype, df: pd.DataFrame, p: SubtypingParams) -> Tuple[Optional[str], Optional[str]]:
    """Are there more missing kmers than tolerated?

    Note:
        For reads, calculate the average coverage depth from the kmers that are present and provide an adequate
        error message based on the coverage.

    Args:
        st: Subtype results
        df: Subtyping results dataframe
        p: Subtyping/QC parameters specifically `max_perc_missing_kmers` for % missing kmers threshold

    Returns:
        None, None if less missing kmers than tolerate; otherwise, "FAIL", error message
    """

    if st.are_subtypes_consistent:
        return check_for_missing_kmers(is_fastq=st.is_fastq_input(),
                                       subtype_result=st.subtype,
                                       scheme=st.scheme,
                                       df=df,
                                       exp=int(st.n_kmers_matching_all_expected),
                                       obs=int(st.n_kmers_matching_all),
                                       p=p)
    else:
        message_list = []

        subtype_list = st.subtype.split(';')
        n_kmers_matching_expected = st.n_kmers_matching_all_expected.split(';')

        dfpos = df[df.is_pos_kmer]
        mixed_subtype_counts = get_mixed_subtype_kmer_counts(dfpos=dfpos,
                                                             subtype_list=subtype_list)

        kmers_matching_negative = st.n_kmers_matching_negative
        for curr_subtype, exp in zip(subtype_list, n_kmers_matching_expected):
            # We can omit the status because there will be a fail status already from non consistent subtypes.

            obs = mixed_subtype_counts.get(curr_subtype) + int(kmers_matching_negative)
            _, curr_messages = check_for_missing_kmers(is_fastq=st.is_fastq_input(),
                                                       subtype_result=curr_subtype,
                                                       scheme=st.scheme,
                                                       df=df,
                                                       exp=int(exp),
                                                       obs=obs,
                                                       p=p)

            message_list.append(curr_messages)

        error_messages = ' | '.join(filter(None.__ne__, message_list))

        return QC.FAIL, error_messages


def check_for_missing_kmers(is_fastq: bool,
                            subtype_result: str,
                            scheme: str,
                            df: pd.DataFrame,
                            exp: int,
                            obs: int,
                            p: SubtypingParams) -> Tuple[Optional[str], Optional[str]]:
    """Check if there are too many missing kmers

    Also check if the mean kmer coverage depth is above the low coverage threshold.

    Args:
        is_fastq: Is input sample reads?
        subtype_result: Single subtype designation
        scheme: Scheme name
        df: Subtyping results dataframe
        exp: Expected number of kmers that should be found
        obs: Actual observed number of kmers found
        p: Subtyping parameters

    Returns:
        Tuple of QC status and any QC messages
    """
    status = None
    messages = None

    # proportion of missing kmers
    p_missing = (exp - obs) / exp  # type: float
    if p_missing > p.max_perc_missing_kmers:
        status = QC.FAIL
        if is_fastq:
            kmers_with_hits = df[df['is_kmer_freq_okay']]  # type: pd.DataFrame
            depth = kmers_with_hits['freq'].mean()
            if depth < p.low_coverage_depth_freq:
                coverage_msg = f'Low coverage depth ({depth:.1f} < {float(p.low_coverage_depth_freq):.1f} expected); ' \
                               f'you may need more WGS data.'
            else:
                coverage_msg = f'Okay coverage depth ({depth:.1f} >= {float(p.low_coverage_depth_freq):.1f} expected), ' \
                               f'but this may be the wrong serovar or species for scheme "{scheme}"'
            messages = f'{p_missing:.2%} missing kmers; more than {p.max_perc_missing_kmers:.2%} missing ' \
                       f'kmers threshold. {coverage_msg}'
        else:
            messages = f'{p_missing:.2%} missing kmers for subtype "{subtype_result}"; more than ' \
                       f'{p.max_perc_missing_kmers:.2%} missing kmer threshold'

    return status, messages


def is_mixed_subtype(st: Subtype, df: pd.DataFrame, *args) -> Tuple[Optional[str], Optional[str]]:
    """Is the subtype result mixed?

    Note:
        Check for `+` and `-` kmers that are present for the target site above minimum coverage threshold.
        If the subtyping result came back with an inconsistent call, provide an adequate error message to the user.

    Args:
        st: Subtype results
        df: Subtyping results dataframe
        args: unused args

    Returns:
        None, None if not mixed subtype result; otherwise, "FAIL", error message
    """
    if not st.are_subtypes_consistent:
        return QC.FAIL, f'Mixed subtypes found: "{"; ".join(sorted(st.inconsistent_subtypes))}".'
    conflicting_kmers = get_conflicting_kmers(st.subtype, df, st.is_fastq_input())
    if conflicting_kmers is None or conflicting_kmers.shape[0] == 0:
        return None, None

    s = 's' if conflicting_kmers.shape[0] > 1 else ''
    positions = ', '.join(conflicting_kmers['refposition'].astype(str).tolist())
    return QC.FAIL, f'Mixed subtype; the positive and negative kmers were found for ' \
                    f'the same target site{s} {positions} for subtype "{st.subtype}".'


def is_missing_too_many_target_sites(st: Subtype, df: pd.DataFrame, p: SubtypingParams) -> Tuple[
    Optional[str], Optional[str]]:
    """Are there too many missing target sites for an expected subtype?

    Check if there are any refpositions missing from the subtyping scheme in the result.
    If there are missing refpositions, this is a missing target.
    If there are too many missing targets, this result is an Ambiguous Result.

    Args:
        st: Subtype results
        df: Subtyping results dataframe
        p: Subtyping/QC parameters

    Returns:
        None, None if less missing targets than tolerated; otherwise, "FAIL", error message
    """
    if not st.are_subtypes_consistent \
            or st.subtype is None:
        return None, None

    potential_subtypes = st.all_subtypes.split('; ')
    uniq_positions = {y for x in potential_subtypes for y in st.scheme_subtype_counts[x].refpositions}
    missing_targets = [x for x in uniq_positions if (df.refposition == x).sum() == 0]

    exp = int(st.n_kmers_matching_all_expected)
    obs = int(st.n_kmers_matching_all)
    if (exp - obs) / exp <= p.max_perc_missing_kmers and len(missing_targets) >= p.min_ambiguous_kmers:
        return QC.FAIL, f'{QC.AMBIGUOUS_RESULTS_ERROR_3}: There were {len(missing_targets)} missing positions for ' \
                        f'subtype "{st.subtype}".'
    return None, None


def is_missing_downstream_targets(st: Subtype, *args) -> Tuple[Optional[str], Optional[str]]:
    """Are there missing downstream targets?

    Note:
        This method will check if there's any non_present_subtypes in the result, which would indicate a non confident
        result. This is due to the fact if you have a subtyping result of `2.1.1.2` and you're missing `2.1.1.2.X`
        You can't be sure that the subtype's final call is 2.1.1.2 if you're missing information.

    Args:
        st: Subtype results
        args: unused extra args

    Returns:
        None, None if no missing downstream targets; otherwise, "FAIL", error message
    """
    if st.non_present_subtypes:
        return QC.FAIL, f'{QC.UNCONFIDENT_RESULTS_ERROR_4}: Subtype "{st.subtype}" was found, but kmers for ' \
                        f'downstream subtype(s) "{st.non_present_subtypes}" were missing. Due to missing downstream ' \
                        f'kmers, there is a lack of confidence in the final subtype call.'
    return None, None


def is_missing_hierarchical_kmers(st: Subtype, *args) -> Tuple[Optional[str], Optional[str]]:
    """Are there any missing nested subtypes in the final subtype call?

    Note:
        This method will check if there's any missing_nested_subtypes in the result, which would indicate a non confident
        result. This is due to the fact that if you have a subtyping result of `2.1.1.2` and you're missing `2.1.1` or `2.1`
        that you can't be sure that the subtype's final call is 2.1.1.2 due to the missing information.

    Args:
        st: Subtype results
        args: unused extra args

    Returns:
        None, None if no missing downstream targets; otherwise, "FAIL", error message
    """
    if st.missing_nested_subtypes:
        return QC.FAIL, f'{QC.UNCONFIDENT_RESULTS_ERROR_4}: Subtype "{st.subtype}" was found, but kmers for ' \
                        f'nested hierarchical subtype(s) "{st.missing_nested_subtypes}" were missing. Due to missing ' \
                        f'kmers, there is a lack of confidence in the final subtype call.'
    return None, None


def is_maybe_intermediate_subtype(st: Subtype, df: pd.DataFrame, p: SubtypingParams) -> Tuple[
    Optional[str], Optional[str]]:
    """Is the result a possible intermediate subtype?

    Return a WARNING message if all the conditions are true:
    - 95% of the scheme kmers are found
    - 0 conflicting kmers
    - total subtype kmers < expected
    - +/- kmers exist in the result.

    Args:
        st: Subtype results
        df: Subtyping results dataframe
        p: Subtyping/QC parameters specifically using `max_perc_intermediate_kmers`

    Returns:
        None, None if no intermediate subtype possible; otherwise, "FAIL", error message
    """
    if not st.are_subtypes_consistent \
            or st.subtype is None:
        return None, None

    total_subtype_kmers = int(st.n_kmers_matching_subtype_expected)
    total_subtype_kmers_hits = int(st.n_kmers_matching_subtype)
    conflicting_kmers = get_conflicting_kmers(st.subtype, df, st.is_fastq_input())
    num_pos_kmers, num_neg_kmers = get_num_pos_neg_kmers(st, df)
    obs = int(st.n_kmers_matching_all)
    exp = int(st.n_kmers_matching_all_expected)
    if (exp - obs) / exp <= p.max_perc_intermediate_kmers and conflicting_kmers.shape[0] == 0 and \
            total_subtype_kmers_hits < total_subtype_kmers and num_pos_kmers and num_neg_kmers:
        return QC.WARNING, f'Possible intermediate subtype. All scheme kmers were found, but a fraction ' \
                           f'were positive for the final subtype. Total subtype matches observed ' \
                           f'(n={total_subtype_kmers_hits}) vs expected (n={total_subtype_kmers})'
    return None, None



# -*- coding: utf-8 -*-

from collections import defaultdict
from itertools import product
import logging
import sys

from ahocorasick import Automaton
import pandas as pd

from ..parsers import parse_fasta, parse_fastq
from ..utils import revcomp
from ..const import bases_dict
from ..subtyping_params import SubtypingParams

def expand_degenerate_bases(seq):
    """List all possible kmers for a scheme given a degenerate base
   
    Args:
         Scheme_kmers from SNV scheme fasta file

    Returns:
         List of all possible kmers given a degenerate base or not
    """

    return list(map("".join, product(*map(bases_dict.get, seq))))

def check_total_kmers(scheme_fasta, subtyping_params):
    '''Checks that the number of kmers about to be created is not at too high a computation or time cost

    Args:
         scheme_fasta: Kmer sequences from the SNV scheme
         Subtyping parameter max_degenerate_kmers: The max kmers allowed by the scheme
    
    Returns:
         None if created kmers < max degenerate kmers argument
         Exits code with warning if created kmers > max degenerate kmers argument
    '''
    kmer_number = 0
    for header, sequence in parse_fasta(scheme_fasta):
        value = 1
        for char in sequence:
            length_key = len(bases_dict[char])
            value = value * length_key
        kmer_number = kmer_number + value
    if kmer_number*2 > subtyping_params.max_degenerate_kmers:
        return logging.error(
            '''
    Your current scheme contains "{}" kmers which is over the current max-degenerate-kmers check of "{}" (Maximum recommended k-mers is 100000).
    It is not advised to run this scheme due to the time and memory usage required to give an output with this many kmers loaded.
    If you still want to run this scheme, add the command line check of "--max-degenerate-kmers {}" at the end of your previous command.
            '''.format(
                kmer_number*2,
                subtyping_params.max_degenerate_kmers,
                kmer_number*2+1
                )), sys.exit()
    else:
        return None


def init_automaton(scheme_fasta):
    """Initialize Aho-Corasick Automaton with kmers from SNV scheme fasta

    Args:
        scheme_fasta: SNV scheme fasta file path

    Returns:
         Aho-Corasick Automaton with kmers loaded
    """
    A = Automaton()
    for header, sequence in parse_fasta(scheme_fasta):
        kmer_list = expand_degenerate_bases(sequence)
        for seq in kmer_list:
            A.add_word(seq, (header, seq, False))
            A.add_word(revcomp(seq), (header, seq, True))
    A.make_automaton()
    return A


def find_in_fasta(A: Automaton, fasta: str) -> pd.DataFrame:
    """Find scheme kmers in input fasta file

    Args:
        A: Aho-Corasick Automaton with scheme SNV target kmers loaded
        fasta: Input fasta path

    Returns:
        Dataframe with any matches found in input fasta file
    """
    res = []
    for contig_header, sequence in parse_fasta(fasta):
        for idx, (kmername, kmer_seq, is_revcomp) in A.iter(sequence):
            res.append((kmername, kmer_seq, is_revcomp, contig_header, idx))
    df = pd.DataFrame(res, columns=['kmername', 'seq', 'is_revcomp', 'contig_id', 'match_index'])
    return df


def find_in_fastqs(A: Automaton, *fastqs):
    """Find scheme kmers in input fastq files

    Args:
        A: Aho-Corasick Automaton with scheme SNV target kmers loaded
        fastqs: Input fastq file paths

    Returns:
        Dataframe with any matches found in input fastq files
    """
    kmer_seq_counts = defaultdict(int)
    for fastq in fastqs:
        for _, sequence in parse_fastq(fastq):
            for idx, (_, kmer_seq, _) in A.iter(sequence):
                kmer_seq_counts[kmer_seq] += 1
    res = []
    for kmer_seq, freq in kmer_seq_counts.items():
        kmername, sequence, _ = A.get(kmer_seq)
        res.append((kmername, kmer_seq, freq))
    df = pd.DataFrame(res, columns=['kmername', 'seq', 'freq'])
    return df



from typing import Tuple, Optional, List, Any, Dict, Iterable

from pandas import DataFrame

from ..subtype import Subtype


def component_subtypes(subtype: str) -> Iterable[str]:
    """Generate component subtypes from a subtype.

    Args:
        subtype: Subtype string, e.g. "4.2.1.1"
    Yields:
        Component subtypes (e.g. for subtype "4.2.1.1", will yield
        ['4', '4.2', '4.2.1', '4.2.1.1'])
    """
    split_subtype = subtype.split('.')
    for i, x in enumerate(split_subtype):
        yield '.'.join(split_subtype[:i+1])


def get_conflicting_kmers(subtype: str, df: DataFrame, is_fastq_input: bool = True) -> Optional[DataFrame]:
    """ Get positive and negative kmers that both are present for a subtype.

    Find positive and negative kmers for the same refposition/target site in the results `df`.

    Args:
        st: Subtype results summary
        df: Detailed subtyping results

    Returns:
        DataFrame of conflicting positive and negative kmers
    """
    dfst = df[(df['subtype'].isin(list(component_subtypes(subtype))))]
    if is_fastq_input:
        dfst = dfst[dfst['is_kmer_freq_okay']]

    pos_kmer_positions = dfst[dfst['is_pos_kmer']]['refposition']
    neg_kmers = dfst[~dfst['is_pos_kmer']]
    return neg_kmers[neg_kmers['refposition'].isin(pos_kmer_positions)]


def get_num_pos_neg_kmers(st: Subtype, df: DataFrame) -> Tuple[int, int]:
    """Get the number of positive and negative kmers for a given subtype

    Args:
        st: Subtype results summary
        df: Detailed subtyping results

    Returns:
        (number of positive kmers, number of negative kmers)
    """
    dfst = df[(df['subtype'] == str(st.subtype))]
    return dfst[dfst['is_pos_kmer']].shape[0], dfst[~dfst['is_pos_kmer']].shape[0]


def get_mixed_subtype_kmer_counts(dfpos: DataFrame, subtype_list: List[Any]) -> Dict[str, int]:
    st_pos_kmers = {}

    for st in subtype_list:
        bs = dfpos.subtype == [st[:x] for x in dfpos.subtype.str.len()]
        st_pos_kmers[st] = bs.sum()

    return st_pos_kmers



# -*- coding: utf-8 -*-

import re
from pkg_resources import resource_filename

from bio_hansel import program_name
from bio_hansel.subtyping_params import SubtypingParams

SCHEME_FASTAS = {'heidelberg': {'file': resource_filename(program_name, 'data/heidelberg/kmers.fasta'),
                                'version': '0.5.0',
                                'subtyping_params': SubtypingParams(low_coverage_depth_freq=20)},
                 'enteritidis': {'file': resource_filename(program_name, 'data/enteritidis/kmers.fasta'),
                                 'version': '1.0.7',
                                 'subtyping_params': SubtypingParams(low_coverage_depth_freq=20)},
                 'typhi': {'file': resource_filename(program_name, 'data/typhi/kmers.fasta'),
                                 'version': '1.2.0',
                                 'subtyping_params': SubtypingParams(low_coverage_depth_freq=20)},
                 'tb_lineage': {'file': resource_filename(program_name, 'data/tb_lineage/kmers.fasta'),
                                 'version': '1.0.5',
                                 'subtyping_params': SubtypingParams(low_coverage_depth_freq=20)},
                 'typhimurium': {'file': resource_filename(program_name, 'data/typhimurium/kmers.fasta'),
                                 'version': '0.5.5',
                                 'subtyping_params': SubtypingParams(low_coverage_depth_freq=20)}}


bases_dict = {
'A': ['A'],
'C': ['C'],
'G': ['G'],
'T': ['T'],
'R': ['A', 'G'],
'Y': ['C', 'T'],
'S': ['G', 'C'],
'W': ['A', 'T'],
'K': ['G', 'T'],
'M': ['A', 'C'],
'B': ['C', 'G', 'T'],
'D': ['A', 'G', 'T'],
'H': ['A', 'C', 'T'],
'V': ['A', 'C', 'G'],
'N': ['A', 'C', 'G', 'T'],}


COLUMNS_TO_REMOVE = '''
pident
length
mismatch
gapopen
qstart
qend
sstart
send
evalue
bitscore
qlen
slen
sseq
is_trunc
coverage
'''.strip().split('\n')

# These are present within the subtype module.
SUBTYPE_SUMMARY_COLS = """
sample
scheme
scheme_version
subtype
all_subtypes
kmers_matching_subtype
are_subtypes_consistent
inconsistent_subtypes
n_kmers_matching_all
n_kmers_matching_all_expected
n_kmers_matching_positive
n_kmers_matching_positive_expected
n_kmers_matching_subtype
n_kmers_matching_subtype_expected
file_path
avg_kmer_coverage
qc_status
qc_message
""".strip().split('\n')


SIMPLE_SUMMARY_COLS = """
sample
subtype
coverage
qc_status
qc_message
""".strip().split('\n')

REGEX_FASTQ = re.compile(r'^(.+)\.(fastq|fq|fastqsanger)(\.gz)?$')
REGEX_FASTA = re.compile(r'^.+\.(fasta|fa|fna|fas)(\.gz)?$')

JSON_EXT_TMPL = '{}.json'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements=['numpy>=1.12.1',
              'pandas>=0.20.1',
              'pyahocorasick>=1.1.6',
              'attrs']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author='Peter Kruczkiewicz',
    author_email='peter.kruczkiewicz@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering'
    ],
    description='Subtype microbial whole-genome sequencing (WGS) data using SNV targeting k-mer subtyping schemes.',
    entry_points={'console_scripts': ['hansel=bio_hansel.main:main']},
    install_requires=requirements,
    keywords='Salmonella enterica Heidelberg Enteritidis SNP kmer subtyping Aho-Corasick',
    license='Apache Software License 2.0',
    long_description=readme,
    name='bio_hansel',
    package_data={'bio_hansel': ['data/*/*.fasta', 'data/*/*.tsv',]},
    packages=find_packages(exclude=['test_*.py', 'tests']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/phac-nml/biohansel',
    version='2.4.0',
    zip_safe=False,
)
